/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.order.manager.service;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bluestemsoftware.open.eoa.example.order.manager.service.OrderProcessor;
import org.bluestemsoftware.open.eoa.example.order.manager.service.dao.CustomerOrderDAO;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.CustomerOrder;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.CustomerOrder.Status;
import org.bluestemsoftware.open.eoa.example.order.manager.util.Constants;
import org.bluestemsoftware.specification.eoa.application.spring.MyOperation;
import org.bluestemsoftware.specification.eoa.component.intrface.rt.ActionContext;
import org.bluestemsoftware.specification.eoa.component.message.rt.Message;

/**
 * Models 'warehouse manager' partner application abstractly, i.e. it handles asynchronous
 * responses from all warehouse locations.
 */
public class AbstractWarehouse implements Warehouse {
    
    private static Log log = LogFactory.getLog(AbstractWarehouse.class);

    private OrderProcessor orderProcessor;
    private CustomerOrderDAO customerOrderDAO;
    
    /**
     * Receives schedule order response asynchronously and terminates message exchange
     * initiated within {@link OrderProcessor#placeOrder(MyOperation, ActionContext)}
     * 
     * @param response
     */
    public void scheduleOrderResponse(ActionContext responseContext) {
        
        // locate order which has been sucessfully scheduled for
        // shipment and update it with estimated ship date

        Message response = responseContext.getMessage();
        String orderNumber = response.evaluateString(Constants.ORDER_NUMBER);
        
        CustomerOrder customerOrder = customerOrderDAO.selectCustomerOrder(orderNumber);
        if (customerOrder == null) {
            log.error("received schedule order response with invalid order id " + orderNumber);
            return;
        }
        
        String temp = response.getContent().getAttribute("estimatedShipDate");
        Date estimatedShipDate = null;
        try {
            estimatedShipDate = DateFormat.getDateInstance().parse(temp);
        } catch (ParseException pe) {
            String error = "error processing schedule order response " + pe.getMessage();
            log.error(error);
            customerOrder.setError(error);
            customerOrder.setStatus(Status.ERROR);
            customerOrderDAO.updateCustomerOrder(customerOrder);
            return;
        }
        
        customerOrder.setEstShipDate(estimatedShipDate);
        customerOrderDAO.updateCustomerOrder(customerOrder);
        
        // notify order processor of estimated ship date, so that it
        // can process the outstanding place order request
        
        orderProcessor.placeOrderResponse(customerOrder, estimatedShipDate);
        
    }
    
    /**
     * Receives get order status response asynchronously.
     * 
     * @param response
     */
    public void getOrderStatusResponse(ActionContext responseContext) {
        // getOrderStatus operation is never invoked on partner
    }
    
    /*
     * dependency injector. set by spring framework
     */
    public void setCustomerOrderDAO(CustomerOrderDAO customerOrderDAO) {
        this.customerOrderDAO = customerOrderDAO;
    }
    
    /*
     * dependency injector. set by spring framework
     */
    public void setOrderProcessor(OrderProcessor orderProcessor) {
        this.orderProcessor = orderProcessor;
    }

}
